
#ifndef SIS_WRAPPER_H_
#define SIS_WRAPPER_H_
#include <linux/fs.h>
#include <linux/mm.h>

#undef  __io_virt_debug
#define __io_virt_debug(a, f, l) (a)

#define OSS_GFP_KERNEL		0x01
#define OSS_GFP_DMA		0x02

#define WRAPPER_VERSION  1
typedef struct {int dont_try_to_use_this_field;}wait_handle;
typedef struct {int dont_try_to_use_this_field;}poll_table_handle, select_table_handle;
typedef struct {int dont_try_to_use_this_field;}pci_handle;
typedef struct {int dont_try_to_use_this_field;}file_handle;
typedef struct {int dont_try_to_use_this_field;}vm_area_handle;
typedef struct {int dont_try_to_use_this_field;}page_handle;
typedef struct {int dont_try_to_use_this_field;}inode_handle;
typedef struct {int dont_try_to_use_this_field;}dentry_handle;
typedef struct {int dont_try_to_use_this_field;}ac97_codec_handle;
typedef struct {int dont_try_to_use_this_field;}module_handle;
typedef struct {int dont_try_to_use_this_field;}dma_addr_handle;
typedef struct {int dont_try_to_use_this_field;}semaphore_handle;
typedef struct {int dont_try_to_use_this_field;}phandle;
typedef struct {int dont_try_to_use_this_field;}task_handle;

extern void * sis_pci_alloc_consistent(void *,size_t,dma_addr_handle *);

struct pci_driver_handle  {
	char * name;
	const struct pci_device_id *id_table;
	int (*probe) (void * pdev,const struct pci_device_id *);
	void (*remove) (void * pdev);
	int	(*suspend) (void * pdev,u32);
	int (*resume) (void * pdev);
};

struct driver_operations_handle {
    char		*name;
    void		*(*attach) (void *loc);
    void		(*suspend) (void *dev);
    void		(*resume) (void *dev);
    void		(*detach) (void *dev);

};

struct ac97_operation_handle {
	u16 (*ac97_read_t) (ac97_codec_handle *,u8);
	void (*ac97_write_t) (ac97_codec_handle *,u16,u8);
};

struct file_operation_handle {
#if LINUX_VERSION_CODE >= 132096 /*2.4.x*/
 	module_handle *owner;
#endif
#if LINUX_VERSION_CODE > 131328 /* 2.1.x */
	long long (*lseek) (file_handle *, long long, int);
#else
	int (*lseek) (inode_handle *, file_handle *, off_t, int);
#endif
#if LINUX_VERSION_CODE > 131328 /* 2.1.x */
	ssize_t (*read) (file_handle *, char *, size_t, loff_t *);
#else
	int (*read) (inode_handle *, file_handle *, char *, int);
#endif
#if LINUX_VERSION_CODE > 131328 /* 2.1.x */
	ssize_t (*write) (file_handle *, const char *, size_t, loff_t *);
#else
	int (*write) (inode_handle *, file_handle *, const char *, int);
#endif
	int (*readdir) (inode_handle *, file_handle *, void *, filldir_t);
#if LINUX_VERSION_CODE > 131328 /* 2.1.x */
	unsigned int (*poll) (file_handle *, poll_table_handle *);
#else
	int (*select) (inode_handle *, file_handle *, int, select_table_handle *);
#endif
	int (*ioctl) (inode_handle *, file_handle *, unsigned int, unsigned long);
#if LINUX_VERSION_CODE > 131328 /* 2.1.x */
	int (*mmap) (file_handle *, vm_area_handle *);
#else
	int (*mmap) (inode_handle *, file_handle *, vm_area_handle *);
#endif
	int (*open) (inode_handle *, file_handle *);
#if LINUX_VERSION_CODE > 131328 /* 2.1.x */
	int (*release) (inode_handle *, file_handle *);
#else
	void (*release) (inode_handle *, file_handle *);
#endif
	int (*fsync) (inode_handle *, file_handle *);
	int (*fasync) (inode_handle *, file_handle *, int);
#if LINUX_VERSION_CODE<131858 /* Prior 2.3.x */
	int (*check_media_change) (kdev_t dev);
	int (*revalidate) (kdev_t dev);
#endif
}file_operation_handle;
///////////////////////////////////////////////////////
unsigned long sis_pci_resource_start(void *,int);
int sis_pci_enable_device(void *dev);
void sis_pci_set_master(void *pci_dev);
unsigned int sis_pcidev_get_irq(void *dev);
int sis_pci_request_regions(void *, char *);
void sis_pci_release_regions(void *);
void sis_pci_set_drvdata(void *ph,void *data);
void * sis_pci_get_drvdata(void *ph);

/////////////////////////////////////////////////////
void sis_netif_device_attach(void *ndev);
int sis_netif_rx(void *skb);
void sis_netdev_set_irq(void *dev,unsigned int irq);
void sis_netdev_set_last_rx(void *dev);

void * sis_netdev_get_priv(void *dev);
char * sis_netdev_get_name(void *dev);
unsigned short sis_netdev_get_flags(void *dev);
void sis_netdev_set_mtu(void *dev,unsigned mtu );
unsigned int sis_netdev_get_irq(void *dev);
unsigned char * sis_netdev_get_dev_addr(void *dev);
void sis_netdev_set_priv(void *dev,void *priv);
void sis_netdev_set_ioctl(void *dev,void *p);
void sis_netdev_set_mlist(void *dev,void *mlist);
void sis_netdev_set_gstats(void *dev,void *p);
void sis_netdev_set_stop(void *dev,void *stop);
void sis_netdev_set_xmit(void *dev,void *xmit);
void sis_netdev_set_open(void *dev,void *open);
void sis_netdev_set_gwstats(void *dev,void *p);
void sis_netdev_set_mtufun(void *dev,void *p);
void sis_netdev_set_maddr(void *dev,void *p);
void * sis_netdev_get_priv(void *dev);
void sis_unregister_netdev(void *dev);
void sis_netdev_set_trans_start(void *dev);
//////////////////////////////////////////////////////////
int sis_netif_running(void *dev);
void sis_netif_start_queue(void *dev);
void sis_netif_stop_queue(void *dev);
////////////////////////////////////////////////////
void sis_set_module_owner(void *ndev);
int sis_pci_module_init(void);
void sis_pci_unregister_driver(void);

////////////////////////////////////////////////////////////////
void sis_set_rx(void *,unsigned long count);
void sis_set_tx(void *,unsigned long count);
void sis_set_rxdrop(void *,unsigned long count);
void sis_set_rxerr(void *,unsigned long count);
void sis_set_txerr(void *,unsigned long count);
void sis_set_rxcrcerr(void *,unsigned long count);
void sis_set_rxbytes(void *,unsigned long count);
void sis_set_txbytes(void *,unsigned long count);
void *sis_get_status(
	unsigned long rxp,
	unsigned long txp,
	unsigned long rxd,
	unsigned long rxe,
	unsigned long txe,
	unsigned long rxcrc,
	unsigned long txb,
	unsigned long rxb
);
void sis_init_status(void *);
void sis_free_status(void *);
/////////////////////////////////////////////////////////////
void sis_dev_kfree_skb_irq(void *skb);
void sis_dev_kfree_skb_any(void *skb);
void * sis_dev_alloc_skb(unsigned int);
unsigned char * sis_skb_put(void *, unsigned int);
unsigned int sis_skb_get_len(void *);
unsigned char * sis_skb_get_data(void *);
void sis_skb_set_dev(void *skb,void * dev);
void sis_skb_set_protocol(void *skb,unsigned short);
void sis_skb_set_ip_summed(void *skb,unsigned char);
////////////////////////////////////////////////////////////
void * sis_memset(void *,int ,size_t);
void * sis_memcpy(void *,void *,size_t);
void * sis_kmalloc(unsigned long,int);
void sis_kfree(void *);
//////////////////////////////////////////////////////
void sis_udelay(unsigned long delay);
/////////////////////////////////////////////////
//#define sis_printk(fmt,arg...) 		\
//{					\
//	printk(fmt,##arg);		\
//	printk("\n");			\
//}	
int sis_printk(const char *);
////////////////////////////////////////////////
unsigned short sis_eth_type_trans(void *skb, void *dev);
//////////////////////////////////////////////////////////
void sis_spin_lock_init(void *);
void sis_spin_lock_free(void *);
void sis_spin_lock(void *);
void sis_spin_unlock(void *);
void sis_spin_lock_irqsave(void *,unsigned long *);
void sis_spin_unlock_irqrestore(void *,unsigned long *);
/////////////////////////////////////////////////////////////
void sis_tq_set(void *tq,void *routine,void *data);
void sis_queue_task(void *);
void sis_run_task_queue(void); 
void sis_mark_bh(void);
////////////////////////////////////////////////////////////////
void sis_timer_init(void *timer,void *timerfun,void *context);
void sis_timer_free(void *timerlist,int count);
void sis_add_timer(void *);
void sis_start_timer(void *,unsigned int);
void sis_timer_exit(void *);
//////////////////////////////////////////////////////////////////
void sis_sleep(unsigned long);
//////////////////////////////////////////////////////////////////
int sis_compare_exchange(long *,unsigned long);
long sis_atomic_add(long *);
long sis_atomic_sub(long *);
//void sis_atomic_set_mask(unsigned long *,unsigned long);
//void sis_atomic_clear_mask(unsigned long *,unsigned long);
//////////////////////////////////////////////////////////////
int sis_usb_register(void);
void sis_usb_deregister(void);
void sis_usb_inc_dev_use(void *dev);
void sis_usb_dec_dev_use(void *dev);
void * sis_usb_alloc_urb(int size);
void sis_usb_free_urb(void *);
void sis_usb_clear_halt(void *usbdev,int piepe);
int sis_usb_submit_urb(void *urb);
void sis_wait_ms(unsigned long ms);
void	sis_rtnl_unlock (void);
void	sis_rtnl_lock (void);
unsigned int sis_usb_rcvbulkpipe(void *usbdev,unsigned char bulk_in_ep);
unsigned int sis_usb_sndbulkpipe(void *usbdev,unsigned char bulk_out_ep);
unsigned int sis_usb_rcvctrlpipe(void *usbdev,unsigned char ctrl_in_ep);
unsigned int sis_usb_sndctrlpipe(void *usbdev,unsigned char ctrl_out_ep);

////////////////////////////////////////////////////////////
#ifdef SIS162_USB
void *sis_fill_device_request(unsigned long requesttype,unsigned long request,unsigned long Index,int size);
void sis_usb_unlink_urb(void *purb);
void SIS_FILL_BULK_URB(void *urb
		,void *usbdev,int pipe
		,char *buffer,int totalLength,
		void *CompleteFun,void *context);

void SIS_FILL_CONTROL_URB(void *urb,void *usbdev,int pipe,void *dr,
			char *buffer,int length,void *CompleteFun,void *context);

void sis_set_thread(void);
void sis_flush_signals(void);
int KeWaitForSingleObject( //keep this as standard wait
				void *EventPollingThread,
				unsigned long Type,
				unsigned long Mode,
				int unused,
				void *TimeoutValue);
void	KeInitializeEvent(
		void *EventPollingThread,
		unsigned long mode, // auto-clearing event
		int type                 // event initially non-signalled
		);
void PsTerminateSystemThread(void *thr_exited);
void KeSetEvent(void *EventPollingThread, unsigned long unused1, int unused2);
int sis_usb_start_wait_urb(void *_urb, int timeout, int* actual_length);
void ctrlcomplete(void *_purb);
void sis_init_completion(void *completion);
void sis_wait_for_completion(void *thr_exited);
int urb_get_status(void *_purb);
int urb_get_actual_length(void *_purb);
void * urb_get_context(void *_purb);
int sis_usb_get_device_descriptor(void *usbdev);
int sis_usb_get_configuration(void *usbdev);
int sis_usb_set_interface(void *usbdev,int ifnum,int mode);
int sis_usb_set_configuration(void *_usbdev,int ifnum);
#endif
void sis_call_usermodehelper(void);
int sis_check_product(void *dev);
/////////////////////////////////////////////////////////////
#define SIS_MODULE_GENERIC_TABLE(gtype,name)	\
static const unsigned long __module_##gtype##_size \
  __attribute__ ((unused)) = sizeof(struct gtype##_id); \
static const struct gtype##_id * __module_##gtype##_table \
  __attribute__ ((unused)) = name
#define SIS_MODULE_DEVICE_TABLE(type,name)		\
  SIS_MODULE_GENERIC_TABLE(type##_device,name)  


#endif
